// This file is part of Lanradio.
//
// Lanradio is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Lanradio is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with Lanradio.  If not, see <http://www.gnu.org/licenses/>.
//
// Authors:
// Florian Klingler <klingler@ccs-labs.org>

#include <veins/modules/application/lanradio/beaconingSchemes/StaticBeaconing.h>

Define_Module(StaticBeaconing);

StaticBeaconing::StaticBeaconing() {

}

StaticBeaconing::~StaticBeaconing() {

}

void StaticBeaconing::initialize(int stage) {

    if(stage == 0) {
        def_beacon_interval = par("def_beacon_interval");
        beacon_offset = par("beacon_offset");

        start_beaconing_at = par("start_beaconing_at");

        classAInt = FindModule<AppInterface*>::findSubModule(getParentModule()); ASSERT(classAInt);

        sendBeaconEvt = new cMessage("classA beacon evt", SEND_STATIC_BEACON_EVT);

        inROI = -1;

        if(simTime() < start_beaconing_at) {
            scheduleAt(start_beaconing_at + beacon_offset*uniform(0,1,1), sendBeaconEvt);
        } else {
            scheduleAt(simTime() + beacon_offset*uniform(0,1,1), sendBeaconEvt);
        }
    }
}

void StaticBeaconing::handleMessage(cMessage *msg) {

    switch (msg->getKind()) {
    case SEND_STATIC_BEACON_EVT: {
        sendBeacon();
        scheduleAt(simTime() + def_beacon_interval, sendBeaconEvt);
        break;
    }
    default: {
        if (msg)
            std::cerr << "ClassA-BeaconAPP: Error: Got Self Message of unknown kind! Name: " << msg->getName() << endl;
        delete msg;
        break;
    }
    }
}

void StaticBeaconing::finish() {
    cancelAndDelete(sendBeaconEvt);
}

void StaticBeaconing::sendBeacon() {
    Enter_Method_Silent();

    if(classAInt->get_is_enabled()) {
        classAInt->sendBeacon();
    }
}

void StaticBeaconing::setInROI(int roi) {
    Enter_Method_Silent();
    inROI = roi;
}

void StaticBeaconing::handleBeacon(WaveShortMessage* msg) {
    Enter_Method_Silent();

}

simtime_t StaticBeaconing::get_next_beacon_time() {
    Enter_Method_Silent();

    return def_beacon_interval;
}

simtime_t StaticBeaconing::get_neighbor_timeout() {
    Enter_Method_Silent();

    return 2*def_beacon_interval;
}
