// This file is part of Lanradio.
//
// Lanradio is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Lanradio is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with Lanradio.  If not, see <http://www.gnu.org/licenses/>.
//
// Authors:
// Florian Klingler <klingler@ccs-labs.org>

#include <veins/modules/mac/lanradio/IntermediateMac.h>
#include <iterator>
#include "veins/modules/messages/PhyControlMessage_m.h"

#if OMNETPP_VERSION >= 0x500
#define OWNER owner->
#else
#define OWNER
#endif

#define DBG_MAC EV
//#define DBG_MAC std::cerr << "[" << simTime().raw() << "] " << myId << " "

Define_Module(IntermediateMac);

void IntermediateMac::initialize(int stage) {
	BaseMacLayer::initialize(stage);

	if(stage == 0) {
	    send_to_lanradio = par("send_to_lanradio").boolValue();
	}
	else if (stage == 1) {
	    lara = LanradioCommunicationAccess().get();
	    if(!lara->registerRadio(this)) {
	        error("lara->registerRadio(this) was unsuccessful");
	    }

	    mac = FindModule<LanradioToMac1609_4Interface*>::findSubModule(
	            getParentModule());
	    assert(mac);
	}
}

void IntermediateMac::handleSelfMsg(cMessage* msg) {

}

void IntermediateMac::handleUpperControl(cMessage* msg) {
	sendControlDown(msg);
}

void IntermediateMac::handleUpperMsg(cMessage* msg) {

    sendDown(msg);
}

void IntermediateMac::handleLowerControl(cMessage* msg) {

    sendControlUp(msg);
}

void IntermediateMac::finish() {

}

void IntermediateMac::handleLowerMsg(cMessage* msg) {

    if(send_to_lanradio) {
        //CAMMessage* thisMsg;
        if (CAMMessage* camMsg = dynamic_cast<CAMMessage*>(msg)) {

            CAM_t cam = camMsg->getCam();
            CAM_t* lanradio_cam = static_cast<CAM_t*>(calloc(1, sizeof(CAM_t)));
            memcpy(lanradio_cam, &cam, sizeof(CAM_t));

            //std::string srcmac = "11:22:33:44:55:66";
            //std::stringstream smac;
            //smac << "11:22:33:44:55:" << msg->getId();
            std::string dstmac = "FF:FF:FF:FF:FF:FF";
            lara->send_WLAN(lanradio_cam, LanradioCommunication::wlan_CAM, camMsg->getSrcmac(), dstmac);

        } else if (SPATMessage* spatMsg = dynamic_cast<SPATMessage*>(msg)) {

            SPAT_PDU_t spat = spatMsg->getSpat();
            SPAT_PDU_t* lanradio_spat = static_cast<SPAT_PDU_t*>(calloc(1, sizeof(SPAT_PDU_t)));
            memcpy(lanradio_spat, &spat, sizeof(SPAT_PDU_t));

            //std::string srcmac = "AA:BB:CC:DD:EE:FF";
            std::string dstmac = "FF:FF:FF:FF:FF:FF";
            lara->send_WLAN(lanradio_spat, LanradioCommunication::wlan_SPAT, spatMsg->getSrcmac(), dstmac);

        } else {
            error("IntermediateMac only accepts CAMMessages and SpatMessages to be sent to Lanradio");
        }
    }
    sendUp(msg);
}

void IntermediateMac::handleLanradioMessage(cMessage* msg) {
    Enter_Method_Silent();

    mac->sendMacPacketDirect(msg);
}
